@@ -1,6 +1,7 @@
 #! /usr/bin/env python
 from __future__ import absolute_import
 import os
+import stat
 import sys
 import socket
 import json
@@ -32,15 +33,31 @@ from irods.test.helpers import (unique_name, my_function_name)
 import irods.parallel
 from irods.manager.data_object_manager import Server_Checksum_Warning
 
-
-def make_ufs_resc_in_tmpdir(session, base_name, allow_local = False):
+def make_ufs_resc_in_tmpdir(session, base_name, allow_local = False, client_vault_mode = (True,)):
+    # Parameters
+    # ----------
+    # base_name         -  The name for the resource, as well as the root directory of the Vault.  Use something random and unlikely to collide.
+    # allow_local       -  Whether to allow the resource's vault to be located under a non-shared ie. "/tmp" style directory.
+    # client_vault_mode -  A tuple of (client_mkdir[, mode_OR_mask]):
+    #                          client_mkdir - whether to call mkdir on the vault-path from the client side, and ...
+    #                          mode_OR_mask - if so, what mode bits to be OR'ed into the permission of the vault path after creation
+    #                                         (A typical value might be : 0o777 | stat.S_ISGID, to guarantee iRODS has permissions on the vault)
     tmpdir = helpers.irods_shared_tmp_dir()
     if not tmpdir and allow_local:
         tmpdir = os.getenv('TMPDIR') or '/tmp'
     if not tmpdir:
         raise RuntimeError("Must have filesystem path shareable with server.")
+
     full_phys_dir = os.path.join(tmpdir,base_name)
-    if not os.path.exists(full_phys_dir): os.mkdir(full_phys_dir)
+
+    if client_vault_mode[0]:
+        if not os.path.exists(full_phys_dir):
+            os.mkdir(full_phys_dir)
+        guarantee_mode_bits = tuple(client_vault_mode[1:])
+        if guarantee_mode_bits != ():
+            mode = os.stat(full_phys_dir).st_mode
+            os.chmod(full_phys_dir, mode | guarantee_mode_bits[0])
+
     session.resources.create(base_name,'unixfilesystem',session.host,full_phys_dir)
     return full_phys_dir
 
@@ -50,6 +67,7 @@ class TestDataObjOps(unittest.TestCase):
 
     from irods.test.helpers import (create_simple_resc)
 
+
     def setUp(self):
         # Create test collection
         self.sess = helpers.make_session()
@@ -1517,6 +1535,81 @@ class TestDataObjOps(unittest.TestCase):
                 self.sess.resources.remove( resc_name )
         self.assertIs( default_XML_parser(), current_XML_parser() )
 
+
+    def test_data_open_on_leaf_is_disallowed__243(self):
+        from irods.exception import DIRECT_CHILD_ACCESS
+        root = unique_name(my_function_name(),datetime.now(),'root')
+        home = helpers.home_collection(self.sess)
+        with self.create_resc_hierarchy(root) as (_ , Leaf):
+            with self.assertRaises(DIRECT_CHILD_ACCESS):
+                self.sess.data_objects.open('{home}/disallowed_243'.format(**locals()), 'w', **{kw.RESC_NAME_KW:Leaf})
+
+    def test_data_open_on_named_resource__243(self):
+        s = self.sess
+        data = s.data_objects
+        home = helpers.home_collection(s)
+        data_name = unique_name(my_function_name(),datetime.now(),'data')
+        resc_name = unique_name(my_function_name(),datetime.now(),'resc')
+        with self.create_simple_resc(resc_name) as resc:
+            data_path = '{home}/{data_name}'.format(**locals())
+            try:
+                with data.open(data_path,'w',**{kw.RESC_NAME_KW:resc}) as f:
+                    f.write(b'abc')
+                d = data.get(data_path)
+                self.assertEqual(len(d.replicas),1)
+                self.assertEqual(d.replicas[0].resource_name, resc)
+            finally:
+                if data.exists(data_path):
+                    data.unlink(data_path, force = True)
+
+
+    class _temporary_resource_adopter:
+        """When used as part of a context block, temporarily adopts the named
+           child resources under the named parent resource.
+        """
+        def __init__(self, sess, parent, child_list=()):
+            self.parent = parent
+            self.child_list = child_list
+            self.sess = sess
+        def __enter__(self):
+            for child in self.child_list:
+                self.sess.resources.add_child(self.parent, child)
+        def __exit__(self, *_):
+            p_obj = self.sess.resources.get(self.parent)
+            for child in set(self.child_list) & set(r.name for r in p_obj.children):
+                self.sess.resources.remove_child(self.parent, child)
+
+
+    def test_access_through_resc_hierarchy__243(self):
+        s = self.sess
+        data_path = '{coll}/{data}'.format(coll = helpers.home_collection(s),
+                                           data = unique_name(my_function_name(),datetime.now()))
+        try:
+            s.resources.create('parent','deferred')
+            with self.create_simple_resc('resc0_243') as r0, \
+                 self.create_simple_resc('resc1_243') as r1, \
+                 self._temporary_resource_adopter(s, parent = 'parent', child_list = (r0, r1)):
+
+                hiers = ['parent;{0}'.format(r) for r in (r0,r1)]
+
+                # Write two different replicas. Although the writing of the second will cause the first to become
+                # stale, each replica can be independently read by specifying the resource hierarchy.
+                for hier in hiers:
+                    opts = {kw.RESC_HIER_STR_KW: hier}
+                    with s.data_objects.open(data_path, 'a', **opts) as obj_io:
+                        obj_io.seek(0)
+                        obj_io.write(hier.encode('utf-8'))   # Write different content to each replica
+
+                # Assert that we are able to read both replicas' content faithfully using the hierarchy string.
+                for hier in hiers:
+                    opts = {kw.RESC_HIER_STR_KW: hier}
+                    with s.data_objects.open(data_path, 'r', **opts) as obj_io:
+                        self.assertEqual(obj_io.read(),hier.encode('utf-8'))   # Does unique replica have unique content?
+
+                s.data_objects.unlink(data_path, force = True)
+        finally:
+            s.resources.remove('parent')
+
     def test_register_with_xml_special_chars(self):
         test_dir = helpers.irods_shared_tmp_dir()
         loc_server = self.sess.host in ('localhost', socket.gethostname())
@@ -1552,6 +1645,52 @@ class TestDataObjOps(unittest.TestCase):
             self.sess.resources.get(resc_name).remove()
 
 
+    def test_unregister_can_target_replica__issue_435(self):
+        test_dir = helpers.irods_shared_tmp_dir()
+        loc_server = self.sess.host in ('localhost', socket.gethostname())
+        if not(test_dir) and not (loc_server):
+            self.skipTest('Requires access to server-side file(s)')
+        dt=datetime.now()
+        uniq1 = unique_name (my_function_name(), 1, dt)
+        uniq2 = unique_name (my_function_name(), 2, dt)
+        dir1 = dir2 = ''
+        data_objects = []
+        try:
+            VAULT_MODE = (loc_server, 0o777|stat.S_ISGID)
+            dir1 = make_ufs_resc_in_tmpdir(self.sess, uniq1, allow_local = loc_server, client_vault_mode = VAULT_MODE)
+            dir2 = make_ufs_resc_in_tmpdir(self.sess, uniq2, allow_local = loc_server, client_vault_mode = VAULT_MODE)
+
+            def replica_number_from_resource_name(data_path, resc):
+                return [r.number for r in self.sess.data_objects.get(data_path).replicas if r.resource_name == resc][0]
+
+            # Use two different ways to specify unregister target:
+            for keyword in (kw.RESC_NAME_KW, kw.REPL_NUM_KW):
+                dt=datetime.now()
+                data_path = '{self.coll_path}/435_test_{dt:%s.%f}'.format(**locals())
+                data = self.sess.data_objects.create(data_path, resource = uniq1)
+                data_objects.append(data)
+
+                # Ensure that two replicas exist.
+                data.replicate(**{kw.DEST_RESC_NAME_KW:uniq2})
+                data = self.sess.data_objects.get(data_path)
+                self.assertEqual(2,len(data.replicas))
+
+                physical_paths = [r.path for r in data.replicas]
+
+                # Assert that unregistering the specific replica decreases the number of replicas by 1.
+                data.unregister(**{keyword:(replica_number_from_resource_name(data_path, uniq2) if keyword == kw.REPL_NUM_KW else uniq2),
+                                   kw.COPIES_KW:1})
+                self.assertEqual(1, len(self.sess.data_objects.get(data_path).replicas))
+
+                # Assert replica files still both on disk.
+                for phys in physical_paths:
+                    os.stat(phys)
+        finally:
+            # Clean up.
+            for d in data_objects: d.unlink(force = True)
+            if dir1: self.sess.resources.get(uniq1).remove()
+            if dir2: self.sess.resources.get(uniq2).remove()
+
 if __name__ == '__main__':
     # let the tests find the parent irods lib
     sys.path.insert(0, os.path.abspath('../..'))
