@@ -14,6 +14,8 @@ import irods.keywords as kw
 from irods.session import iRODSSession
 from six.moves import range
 from six import PY3
+from irods.message import Bad_AVU_Field
+
 
 
 class TestMeta(unittest.TestCase):
@@ -45,7 +47,7 @@ class TestMeta(unittest.TestCase):
 
     def test_atomic_metadata_operations_244(self):
         user = self.sess.users.get("rods")
-        group = self.sess.user_groups.get("public")
+        group = self.sess.groups.get("public")
         m = ( "attr_244","value","units")
 
         with self.assertRaises(BadAVUOperationValue):
@@ -108,7 +110,7 @@ class TestMeta(unittest.TestCase):
     def test_atomic_metadata_operations_255(self):
         my_resc = self.sess.resources.create('dummyResc','passthru')
         avus = [iRODSMeta('a','b','c'), iRODSMeta('d','e','f')]
-        objects = [ self.sess.users.get("rods"), self.sess.user_groups.get("public"), my_resc,
+        objects = [ self.sess.users.get("rods"), self.sess.groups.get("public"), my_resc,
                     self.sess.collections.get(self.coll_path), self.sess.data_objects.get(self.obj_path)  ]
         try:
             for obj in objects:
@@ -442,6 +444,14 @@ class TestMeta(unittest.TestCase):
                 if d: d.unlink(force = True)
                 helpers.remove_unused_metadata(session)
 
+    def test_nonstring_as_AVU_value_raises_an_error__issue_434(self):
+        with self.assertRaisesRegexp(Bad_AVU_Field,'incorrect type'):
+            self.coll.metadata.set("an_attribute",0)
+
+    def test_empty_string_as_AVU_value_raises_an_error__issue_434(self):
+        with self.assertRaisesRegexp(Bad_AVU_Field,'zero-length'):
+            self.coll.metadata.set("an_attribute","")
+
 if __name__ == '__main__':
     # let the tests find the parent irods lib
     sys.path.insert(0, os.path.abspath('../..'))
